@@ -15,8 +15,7 @@ try:
 except ImportError:
     import _clinkgrammar as clg
 
-locale.setlocale(locale.LC_ALL, "en_US.UTF-8")
-
+locale.setlocale(locale.LC_CTYPE, "en_US.UTF-8")
 
 def setUpModule():
     datadir = os.getenv("LINK_GRAMMAR_DATA", "")
@@ -34,11 +33,11 @@ def setUpModule():
 # First test: test the test framework itself ...
 class AAALinkTestCase(unittest.TestCase):
     def test_link_display_with_identical_link_type(self):
-        self.assertEqual(unicode(Link(None, 0, 'Left','Link','Link','Right')),
+        self.assertEqual(str(Link(None, 0, 'Left','Link','Link','Right')),
                          u'Left-Link-Right')
 
     def test_link_display_with_identical_link_type2(self):
-        self.assertEqual(unicode(Link(None, 0, 'Left','Link','Link*','Right')),
+        self.assertEqual(str(Link(None, 0, 'Left','Link','Link*','Right')),
                          u'Left-Link-Link*-Right')
 
 class BParseOptionsTestCase(unittest.TestCase):
@@ -179,7 +178,6 @@ class BParseOptionsTestCase(unittest.TestCase):
         po = ParseOptions(linkage_limit=99)
         self.assertEqual(clg.parse_options_get_linkage_limit(po._obj), 99)
 
-
 class CParseOptionsTestCase(unittest.TestCase):
 
     def test_that_sentence_can_be_destroyed_when_linkages_still_exist(self):
@@ -202,6 +200,11 @@ class DBasicParsingTestCase(unittest.TestCase):
     def parse_sent(self, text):
         return list(Sentence(text, self.d, self.po).parse())
 
+    def test_that_parse_returns_empty_iterator_on_no_linkage(self):
+        result = self.parse_sent("This this doesn't parse");
+        for _ in result:
+            assert False, "Unexpected linkage iteration"
+
     def test_that_parse_sent_returns_list_of_linkage_objects_for_valid_sentence(self):
         result = self.parse_sent("This is a relatively simple sentence.")
         self.assertTrue(isinstance(result[0], Linkage))
@@ -214,7 +217,10 @@ class DBasicParsingTestCase(unittest.TestCase):
         self.assertTrue(isinstance(result[1], Linkage))
 
         # def test_unicode_encoded_string(self):
-        result = self.parse_sent(u"I love going to the caf\N{LATIN SMALL LETTER E WITH ACUTE}.".encode('utf8'))
+        if (sys.version_info > (3, 0)):
+            result = self.parse_sent(u"I love going to the caf\N{LATIN SMALL LETTER E WITH ACUTE}.")
+        else:
+            result = self.parse_sent(u"I love going to the caf\N{LATIN SMALL LETTER E WITH ACUTE}.".encode('utf8'))
         self.assertTrue(1 < len(result))
         self.assertTrue(isinstance(result[0], Linkage))
         self.assertTrue(isinstance(result[1], Linkage))
@@ -243,6 +249,12 @@ class DBasicParsingTestCase(unittest.TestCase):
         linkage = self.parse_sent("This is a silly sentence.")[0]
         self.assertEqual([len(l) for l in linkage.links()], [6,2,1,1,3,2,1,1,1])
 
+    def test_dictionary_locale_definition(self):
+        oldlocale = locale.setlocale(locale.LC_CTYPE, "tr_TR.UTF-8")
+        self.assertEqual(list(self.parse_sent('Is it fine?')[0].words()),
+             ['LEFT-WALL', 'is.v', 'it', 'fine.a', '?', 'RIGHT-WALL'])
+        locale.setlocale(locale.LC_CTYPE, oldlocale);
+
 class ESATsolverTestCase(unittest.TestCase):
     def setUp(self):
         self.d, self.po = Dictionary(lang='en'), ParseOptions()
