@@ -102,22 +102,26 @@ Parse_Options parse_options_create(void)
 	po = (Parse_Options) xalloc(sizeof(struct Parse_Options_s));
 
 	/* Here's where the values are initialized */
-	po->verbosity = 1;
-	po->debug = (char *)"";
-	po->test = (char *)"";
-	po->linkage_limit = 100;
+
+	/* The parse_options_set_(verbosity|debug|test) functions set also the
+	 * corresponding global variables. So these globals are initialized
+	 * here too. */
+	verbosity = po->verbosity = 1;
+	debug = po->debug = (char *)"";
+	test = po->test = (char *)"";
 
 	/* A cost of 2.7 allows the usual cost-2 connectors, plus the
 	 * assorted fractional costs, without going to cost 3.0, which
 	 * is used only during panic-parsing.
 	 */
-	po->disjunct_cost = 2.7; /* 3.0 is needed for Russian dicts */
+	po->disjunct_cost = 2.7;
 	po->min_null_count = 0;
 	po->max_null_count = 0;
 	po->islands_ok = false;
 	po->use_spell_guess = 7;
 	po->use_sat_solver = false;
 	po->use_viterbi = false;
+	po->linkage_limit = 100;
 
 #ifdef XXX_USE_CORPUS
 	/* Use the corpus cost model, if available.
@@ -701,7 +705,7 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 
 	print_time(opts, "Postprocessed all linkages");
 
-	if (opts->verbosity > 1)
+	if (debug_level(6))
 	{
 		err_ctxt ec;
 		ec.sent = sent;
@@ -1047,7 +1051,7 @@ static void wordgraph_path_free(Wordgraph_pathpos *wp, bool free_final_path)
  *
  * Return true if the linkage is good, else return false.
  */
-#define D_SLM 4
+#define D_SLM 7
 bool sane_linkage_morphism(Sentence sent, Linkage lkg, Parse_Options opts)
 {
 	Wordgraph_pathpos *wp_new = NULL;
@@ -1261,8 +1265,9 @@ bool sane_linkage_morphism(Sentence sent, Linkage lkg, Parse_Options opts)
 			/* Notify to stdout, so it will be shown along with the result.
 			 * XXX We should have a better way to notify. */
 			if (0 < opts->verbosity)
-				printf("Warning: Invalid morpheme type combination '%s', "
-				       "run with !bad and !verbosity=4 to debug\n", affix_types);
+				printf("Warning: Invalid morpheme type combination '%s'.\n"
+				       "Run with !bad and !verbosity>"STRINGIFY(D_USER_MAX)
+				       " to debug\n", affix_types);
 		}
 	}
 
@@ -1307,7 +1312,7 @@ static void sane_morphism(Sentence sent, Parse_Options opts)
 			N_invalid_morphism ++;
 	}
 
-	if (opts->verbosity > 1)
+	if (debug_level(5))
 	{
 		prt_error("Info: sane_morphism(): %zu of %zu linkages had "
 		          "invalid morphology construction\n",
@@ -1352,7 +1357,7 @@ static void chart_parse(Sentence sent, Parse_Options opts)
 		hist = do_parse(sent, mchxt, ctxt, sent->null_count, opts);
 		total = hist_total(&hist);
 
-		if (opts->verbosity > 1)
+		if (debug_level(5))
 		{
 			prt_error("Info: Total count with %zu null links:   %lld\n",
 			          sent->null_count, total);
@@ -1403,10 +1408,6 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 {
 	int rc;
 
-	verbosity = opts->verbosity;
-	debug = opts->debug;
-	test = opts->test;
-
 	sent->num_valid_linkages = 0;
 
 	/* If the sentence has not yet been split, do so now.
