@@ -14,6 +14,7 @@
 #define _LINK_GRAMMAR_DISJUNCT_UTILS_H_
 
 #include "api-types.h"
+#include "structures.h"
 
 /* Disjunct utilities ... */
 void free_disjuncts(Disjunct *);
@@ -21,5 +22,7 @@ unsigned int count_disjuncts(Disjunct *);
 Disjunct * catenate_disjuncts(Disjunct *, Disjunct *);
 Disjunct * eliminate_duplicate_disjuncts(Disjunct * );
 char * print_one_disjunct(Disjunct *);
+void word_record_in_disjunct(const Gword *, Disjunct *);
+void disjunct_word_print(Disjunct *);
 
 #endif /* _LINK_GRAMMAR_DISJUNCT_UTILS_H_ */
