@@ -67,7 +67,7 @@ static int right_disjunct_list_length(const Disjunct * d)
  */
 static void push_match_list_element(fast_matcher_t *ctxt, Disjunct *d)
 {
-	if (ctxt->match_list_end > ctxt->match_list_size)
+	if (ctxt->match_list_end >= ctxt->match_list_size)
 	{
 		ctxt->match_list_size *= MATCH_LIST_SIZE_INC;
 		ctxt->match_list = realloc(ctxt->match_list,
@@ -109,7 +109,7 @@ void free_fast_matcher(fast_matcher_t *mchxt)
 	}
 
 	free(mchxt->match_list);
-	lgdebug(5, "Sentence size %zu, match_list_size %zu\n",
+	lgdebug(6, "Sentence size %zu, match_list_size %zu\n",
 	        mchxt->size, mchxt->match_list_size);
 
 	xfree(mchxt->l_table_size, mchxt->size * sizeof(unsigned int));
