@@ -21,34 +21,33 @@
 #include <stdarg.h>
 
 #ifndef _WIN32
-#include <unistd.h>
+	#include <unistd.h>
+	#include <langinfo.h>
+#else
+	#include <windows.h>
+	#include <Shlwapi.h> /* For PathRemoveFileSpecA(). */
+	#include <direct.h>  /* For getcwd(). */
 #endif /* _WIN32 */
 
-
 #ifdef USE_PTHREADS
-#include <pthread.h>
-#endif
+	#include <pthread.h>
+#endif /* USE_PTHREADS */
 
 #include "string-set.h"
 #include "structures.h"
 #include "utilities.h"
 
-#ifdef ENABLE_BINRELOC
-#include "prefix.h"
-#endif /* BINRELOC */
-
 #ifdef _WIN32
-#  include <windows.h>
- #define DIR_SEPARATOR "\\"
+	#define DIR_SEPARATOR "\\"
 #else
- #define DIR_SEPARATOR "/"
-#endif
+	#define DIR_SEPARATOR "/"
+#endif /*_WIN32 */
 
 #define IS_DIR_SEPARATOR(ch) (DIR_SEPARATOR[0] == (ch))
 #ifndef DICTIONARY_DIR
- #define DEFAULTPATH NULL
+	#define DEFAULTPATH NULL
 #else
- #define DEFAULTPATH DICTIONARY_DIR
+	#define DEFAULTPATH DICTIONARY_DIR
 #endif
 
 /* This file contains certain general utilities. */
@@ -117,48 +116,7 @@ void left_print_string(FILE * fp, const char * s, const char * t)
 	fprintf(fp, "%-*s", width, s);
 }
 
-#ifdef _WIN32  /* should be !defined(HAVE_STRTOK_R) */
-
-char *
-strtok_r (char *s, const char *delim, char **saveptr)
-{
-	char *p;
-
-	if (s == NULL)
-		s = *saveptr;
-
-	if (s == NULL)
-		return NULL;
-
-	/* Skip past any delimiters. */
-	/* while (*s && strchr (delim, *s)) s++; */
-	s += strspn(s, delim);
-
-	if (*s == '\0')
-	{
-		*saveptr = NULL;
-		return NULL;
-	}
-
-	/* Look for end of the token. */
-	/* p = s; while (*p && !strchr (delim, *p)) p++; */
-	p = strpbrk(s, delim);
-	if (p == NULL)
-	{
-		*saveptr = NULL;
-		return s;
-	}
-
-	*p = 0x0;
-	*saveptr = p+1;
-
-	return s;
-}
-
-#endif /* _WIN32 should be !HAVE_STROTOK_R */
-
-#ifdef _WIN32  /* should be !defined(HAVE_STRNDUP) */
-
+#ifndef HAVE_STRNDUP
 /* Emulates glibc's strndup() */
 char *
 strndup (const char *str, size_t size)
@@ -177,20 +135,17 @@ strndup (const char *str, size_t size)
 	result[size] = 0x0;
 	return result;
 }
-
-#endif /* _WIN32 should be !HAVE_STRNDUP */
+#endif /* !HAVE_STRNDUP */
 
 /* ============================================================= */
 /* UTF8 utilities */
 
-#if defined(_MSC_VER) || defined(__MINGW32__)
-
 /** Returns length of UTF8 character.
  * Current algo is based on the first character only.
  * If pointer is not pointing at first char, no not a valid value, returns 0.
  * Returns 0 for NULL as well.
  */
-static int get_utf8_charlen(const char *xc)
+int utf8_charlen(const char *xc)
 {
 	unsigned char c;
 
@@ -204,6 +159,7 @@ static int get_utf8_charlen(const char *xc)
 	return -1; /* Fallthrough -- not the first byte of a code-point. */
 }
 
+#ifdef _WIN32
 /**
  * (Experimental) Implementation of mbrtowc for Windows.
  * This is required because the other, commonly available implementations
@@ -218,34 +174,23 @@ size_t lg_mbrtowc(wchar_t *pwc, const char *s, size_t n, mbstate_t *ps)
 	if (0 == n) return -2;
 	if (0 == *s) { *pwc = 0; return 0; }
 
-	nb = get_utf8_charlen(s);
+	nb = utf8_charlen(s);
 	if (0 == nb) return 0;
 	if (0 > nb) return nb;
 	nb2 = MultiByteToWideChar(CP_UTF8, 0, s, nb, NULL, 0);
 	nb2 = MultiByteToWideChar(CP_UTF8, 0, s, nb, pwc, nb2);
-	if (0 == nb2) return 0;
+	if (0 == nb2) return (size_t)-1;
 	return nb;
 }
-#endif /* defined(_MSC_VER) || defined(__MINGW32__) */
-
-#if __APPLE__
-/* Junk, to keep the Mac OSX linker happy, because this is listed in
- * the link-grammar.def symbol export file.  */
-void lg_mbrtowc(void) {}
-#endif
+#endif /* _WIN32 */
 
 static int wctomb_check(char *s, wchar_t wc)
 {
 	int nr;
-#if defined(_MSC_VER) || defined(__MINGW32__)
+#ifdef _WIN32
 	nr = WideCharToMultiByte(CP_UTF8, 0, &wc, 1, NULL, 0, NULL, NULL);
 	nr = WideCharToMultiByte(CP_UTF8, 0, &wc, 1, s, nr, NULL, NULL);
-	if (nr < 0) {
-		wprintf(L"Fatal Error: wctomb_check failed: %d %S\n", nr, &wc);
-		exit(1);
-	}
-	/* XXX TODO Perhaps the below needs to be uncommented?  .. tucker says no ... */
-	/* nr = nr / sizeof(wchar_t); */
+	if (0 == nr) return -1;
 #else
 	mbstate_t mbss;
 	memset(&mbss, 0, sizeof(mbss));
@@ -254,7 +199,7 @@ static int wctomb_check(char *s, wchar_t wc)
 		prt_error("Fatal Error: unknown character set %s\n", nl_langinfo(CODESET));
 		exit(1);
 	}
-#endif
+#endif /* _WIN32 */
 	return nr;
 }
 
@@ -265,29 +210,31 @@ static int wctomb_check(char *s, wchar_t wc)
  * because the byte-counts might not match up, e.g. German ß and SS.
  * The correct long-term fix is to use ICU or glib g_utf8_strup(), etc.
  */
-void downcase_utf8_str(char *to, const char * from, size_t usize)
+void downcase_utf8_str(char *to, const char * from, size_t usize, locale_t locale_t)
 {
 	wchar_t c;
 	int i, nbl, nbh;
 	char low[MB_LEN_MAX];
 	mbstate_t mbs;
 
-	to[0] = '\0'; /* Make sure it doesn't contain garbage in case of an error */
+	/* Make sure it doesn't contain garbage in case of an error */
+	if (to != from) strcpy(to, from);
+
 	memset(&mbs, 0, sizeof(mbs));
 	nbh = mbrtowc (&c, from, MB_CUR_MAX, &mbs);
 	if (nbh < 0)
 	{
-		prt_error("Error: Invalid multi-byte string!");
+		prt_error("Error: Invalid UTF-8 string!");
 		return;
 	}
-	c = towlower(c);
+	c = towlower_l(c, locale_t);
 	nbl = wctomb_check(low, c);
 
 	/* Check for error on an in-place copy */
 	if ((nbh < nbl) && (to == from))
 	{
 		/* I'm to lazy to fix this */
-		prt_error("Error: can't downcase multi-byte string!");
+		prt_error("Error: can't downcase UTF-8 string!");
 		return;
 	}
 
@@ -308,7 +255,7 @@ void downcase_utf8_str(char *to, const char * from, size_t usize)
  * because the byte-counts might not match up, e.g. German ß and SS.
  * The correct long-term fix is to use ICU or glib g_utf8_strup(), etc.
  */
-void upcase_utf8_str(char *to, const char * from, size_t usize)
+void upcase_utf8_str(char *to, const char * from, size_t usize, locale_t locale_t)
 {
 	wchar_t c;
 	int i, nbl, nbh;
@@ -319,17 +266,17 @@ void upcase_utf8_str(char *to, const char * from, size_t usize)
 	nbh = mbrtowc (&c, from, MB_CUR_MAX, &mbs);
 	if (nbh < 0)
 	{
-		prt_error("Error: Invalid multi-byte string!");
+		prt_error("Error: Invalid UTF-8 string!");
 		return;
 	}
-	c = towupper(c);
+	c = towupper_l(c, locale_t);
 	nbl = wctomb_check(low, c);
 
 	/* Check for error on an in-place copy */
 	if ((nbh < nbl) && (to == from))
 	{
 		/* I'm to lazy to fix this */
-		prt_error("Error: can't upcase multi-byte string!");
+		prt_error("Error: can't upcase UTF-8 string!");
 		return;
 	}
 
@@ -392,7 +339,7 @@ static void space_key_alloc(void)
 }
 #else
 static space_t space;
-#endif
+#endif /* USE_PTHREADS */
 
 static space_t * do_init_memusage(void)
 {
@@ -403,7 +350,7 @@ static space_t * do_init_memusage(void)
 	pthread_setspecific(space_key, s);
 #else
 	s = &space;
-#endif
+#endif  /* USE_PTHREADS */
 
 	s->max_space_used = 0;
 	s->space_in_use = 0;
@@ -427,7 +374,7 @@ void init_memusage(void)
 	static bool mem_inited = false;
 	if (mem_inited) return;
 	mem_inited = true;
-#endif
+#endif /* USE_PTHREADS */
 	do_init_memusage();
 }
 
@@ -439,7 +386,7 @@ static inline space_t *getspace(void)
 	return do_init_memusage();
 #else
 	return &space;
-#endif
+#endif /* USE_PTHREADS */
 }
 
 /**
@@ -589,98 +536,6 @@ char * join_path(const char * prefix, const char * suffix)
 	return path;
 }
 
-/* borrowed from glib */
-/* Used only for Windows builds */
-#ifdef _WIN32
-static char*
-path_get_dirname (const char *file_name)
-{
-	register char *base;
-	register int len;
-
-	base = strrchr (file_name, DIR_SEPARATOR[0]);
-#ifdef _WIN32
-	{
-		char *q = strrchr (file_name, '/');
-		if (base == NULL || (q != NULL && q > base))
-			base = q;
-	}
-#endif
-	if (!base)
-	{
-#ifdef _WIN32
-		if (is_utf8_alpha (file_name) && file_name[1] == ':')
-		{
-			char drive_colon_dot[4];
-
-			drive_colon_dot[0] = file_name[0];
-			drive_colon_dot[1] = ':';
-			drive_colon_dot[2] = '.';
-			drive_colon_dot[3] = '\0';
-
-			return safe_strdup (drive_colon_dot);
-		}
-#endif
-		return safe_strdup (".");
-	}
-
-	while (base > file_name && IS_DIR_SEPARATOR (*base))
-		base--;
-
-#ifdef _WIN32
-	/* base points to the char before the last slash.
-	 *
-	 * In case file_name is the root of a drive (X:\) or a child of the
-	 * root of a drive (X:\foo), include the slash.
-	 *
-	 * In case file_name is the root share of an UNC path
-	 * (\\server\share), add a slash, returning \\server\share\ .
-	 *
-	 * In case file_name is a direct child of a share in an UNC path
-	 * (\\server\share\foo), include the slash after the share name,
-	 * returning \\server\share\ .
-	 */
-	if (base == file_name + 1 && is_utf8_alpha (file_name) && file_name[1] == ':')
-		base++;
-	else if (IS_DIR_SEPARATOR (file_name[0]) &&
-	         IS_DIR_SEPARATOR (file_name[1]) &&
-	         file_name[2] &&
-	         !IS_DIR_SEPARATOR (file_name[2]) &&
-	         base >= file_name + 2)
-	{
-		const char *p = file_name + 2;
-		while (*p && !IS_DIR_SEPARATOR (*p))
-			p++;
-		if (p == base + 1)
-		{
-			len = (int) strlen (file_name) + 1;
-			base = (char *)malloc(len + 1);
-			strcpy (base, file_name);
-			base[len-1] = DIR_SEPARATOR[0];
-			base[len] = 0;
-			return base;
-		}
-		if (IS_DIR_SEPARATOR (*p))
-		{
-			p++;
-			while (*p && !IS_DIR_SEPARATOR (*p))
-				p++;
-			if (p == base + 1)
-				base++;
-		}
-	}
-#endif
-
-	len = (int) 1 + base - file_name;
-
-	base = (char *)malloc(len + 1);
-	memmove (base, file_name, len);
-	base[len] = 0;
-
-	return base;
-}
-#endif /* _WIN32 */
-
 /* global - but that's OK, since this is set only during initialization,
  * and is is thenceforth a read-only item. So it doesn't need to be
  * locked.
@@ -695,9 +550,6 @@ void dictionary_set_data_dir(const char * path)
 
 char * dictionary_get_data_dir(void)
 {
-#ifdef _WIN32
-	HINSTANCE hInstance;
-#endif
 	char * data_dir = NULL;
 
 	if (custom_data_dir != NULL) {
@@ -705,75 +557,42 @@ char * dictionary_get_data_dir(void)
 		return data_dir;
 	}
 
-#ifdef ENABLE_BINRELOC
-	data_dir = safe_strdup (BR_DATADIR("/link-grammar"));
-#elif defined(_WIN32)
-	/* Dynamically locate library and return containing directory.
-	 * FIXME: A rewrite is needed. */
-	hInstance = GetModuleHandle("link-grammar.dll");
-	if (hInstance != NULL)
-	{
-		char dll_path[MAX_PATH];
+#ifdef _WIN32
+	/* Dynamically locate invocation directory of our program.
+	 * Non-ASCII characters are not supported (files will not be found). */
+	char prog_path[MAX_PATH_NAME];
 
-		if (GetModuleFileName(hInstance, dll_path, MAX_PATH))
-		{
-#ifdef _DEBUG
-			prt_error("Info: GetModuleFileName=%s", (dll_path ? dll_path : "NULL"));
-#endif
-			data_dir = path_get_dirname(dll_path);
-		}
+	if (!GetModuleFileNameA(NULL, prog_path, sizeof(prog_path)))
+	{
+		prt_error("Warning: GetModuleFileName error %d", (int)GetLastError());
 	}
 	else
 	{
-		/* BHayes added else block for when link-grammar.dll is not found 11apr11 */
-		/* This requests module handle for the current program */
-		hInstance = GetModuleHandle(NULL);
-		if (hInstance != NULL)
+		if (NULL == prog_path)
 		{
-			char prog_path16[MAX_PATH];
-			printf("  found ModuleHandle for current program so try to get Filename for current program \n");
-			if (GetModuleFileName(hInstance, prog_path16, MAX_PATH))
+			/* Can it happen? */
+			prt_error("Warning: GetModuleFileName returned a NULL program path!");
+		}
+		else
+		{
+			if (!PathRemoveFileSpecA(prog_path))
 			{
-				char * data_dir16 = NULL;
+				prt_error("Warning: Cannot get directory from program path '%s'!",
+				          prog_path);
+			}
+			else
+			{
+				/* Unconvertible characters are marked as '?' */
+				const char *unsupported = (NULL != strchr(prog_path, '?')) ?
+					" (containing unsupported character)" : "";
 
-				if (sizeof(TCHAR) == 1)
-				{
-				    data_dir16 = path_get_dirname(prog_path16);
-				}
-				else
-				{
-					// convert 2-byte to 1-byte encoding of prog_path
-					char prog_path[MAX_PATH/2];
-
-					int i, k;
-					for (i = 0; i < MAX_PATH; i++)
-					{
-						k = i + i;
-						if (prog_path16[k] == 0 )
-						{
-							prog_path[i] = 0;
-							break; // found the string ending null char
-						}
-						// XXX this cannot possibly be correct for UTF-16 filepaths!! ?? FIXME
-						prog_path[i] = prog_path16[k];
-					}
-#ifdef _DEBUG
-					prt_error("Info: GetModuleFileName=%s", (prog_path ? prog_path : "NULL"));
-#endif
-				    data_dir16 = path_get_dirname(prog_path);
-				}
-				if (data_dir16 != NULL)
-				{
-					printf("   found dir for current prog %s\n", data_dir16);
-				}
-				return data_dir16;  // return path of data directory here instead of below
-			} else {
-				printf("   FAIL GetModuleFileName for current program \n");
+				lgdebug(D_USER_FILES, "Debug: Directory of executable: %s%s\n",
+				        unsupported, prog_path);
+				data_dir = safe_strdup(prog_path);
 			}
 		}
-   }
-#endif
-
+	}
+#endif /* _WIN32 */
 
 	return data_dir;
 }
@@ -802,6 +621,7 @@ char * dictionary_get_data_dir(void)
  * If the filename argument is NULL, the function just invalidates this
  * directory path cache.
  */
+#define NOTFOUND(fp) ((NULL == (fp)) ? " (Not found)" : "")
 void * object_open(const char *filename,
                    void * (*opencb)(const char *, const void *),
                    const void * user_data)
@@ -823,9 +643,14 @@ void * object_open(const char *filename,
 	if (NULL == path_found)
 	{
 		data_dir = dictionary_get_data_dir();
-#ifdef _DEBUG
-		prt_error("Info: data_dir=%s", (data_dir?data_dir:"NULL"));
-#endif
+		if (debug_level(D_USER_FILES))
+		{
+			char cwd[MAX_PATH_NAME];
+			char *cwdp = getcwd(cwd, sizeof(cwd));
+			printf("Debug: Current directory: %s\n", NULL == cwdp ? "NULL": cwdp);
+			printf("Debug: Last-resort data directory: %s\n",
+					  data_dir ? data_dir : "NULL");
+		}
 	}
 
 	/* Look for absolute filename.
@@ -835,17 +660,16 @@ void * object_open(const char *filename,
 	 * seems to use forward-slash, from what I can tell.
 	 */
 	if ((filename[0] == '/')
-#if defined(_WIN32) || defined(_MSC_VER) || defined(__MINGW32__)
+#ifdef _WIN32
 		|| ((filename[1] == ':')
 			 && ((filename[2] == '\\') || (filename[2] == '/')))
-#endif
+		|| (filename[0] == '\\') /* UNC path */
+#endif /* _WIN32 */
 	   )
 	{
 		/* opencb() returns NULL if the file does not exist. */
 		fp = opencb(filename, user_data);
-#ifdef _DEBUG
-		prt_error("Info: 1: object_open() trying %s=%d", filename, NULL!=fp);
-#endif
+		lgdebug(D_USER_FILES, "Debug: Opening file %s%s\n", filename, NOTFOUND(fp));
 	}
 	else
 	{
@@ -870,9 +694,7 @@ void * object_open(const char *filename,
 			free(completename);
 			completename = join_path(*path, filename);
 			fp = opencb(completename, user_data);
-#ifdef _DEBUG
-			prt_error("Info: 2: object_open() trying %s=%d", completename, NULL!=fp);
-#endif
+			lgdebug(D_USER_FILES, "Debug: Opening file %s%s\n", completename, NOTFOUND(fp));
 			if ((NULL != fp) || (NULL != path_found)) break;
 		}
 	}
@@ -880,30 +702,27 @@ void * object_open(const char *filename,
 	if (NULL == fp)
 	{
 		fp = opencb(filename, user_data);
-#ifdef _DEBUG
-		prt_error("Info: 3: object_open() trying %s=%d", filename, NULL!=fp);
-#endif
+		lgdebug(D_USER_FILES, "Debug: Opening file %s%s\n", filename, NOTFOUND(fp));
 	}
 	else if (NULL == path_found)
 	{
 		size_t i;
 
 		path_found = strdup((NULL != completename) ? completename : filename);
-		prt_error("Info: Dictionary found at %s", path_found);
+		if (0 < verbosity)
+			prt_error("Info: Dictionary found at %s", path_found);
 		for (i = 0; i < 2; i++)
 		{
 			char *root = strrchr(path_found, DIR_SEPARATOR[0]);
 			if (NULL != root) *root = '\0';
 		}
-#ifdef _DEBUG
-		prt_error("Info: object_open() path_found=%s", path_found);
-#endif
 	}
 
 	free(data_dir);
 	free(completename);
 	return fp;
 }
+#undef NOTFOUND
 
 static void *dict_file_open(const char *fullname, const void *how)
 {
@@ -926,13 +745,10 @@ bool file_exists(const char * dict_name)
 	int fd;
 	struct stat buf;
 
-#if defined(_MSC_VER) || defined(__MINGW32__)
-	/* binary, otherwise fstat file length is confused by crlf
-	 * counted as one byte. */
+	/* On Windows, 'b' (binary mode) is mandatory, otherwise fstat file length
+	 * is confused by crlf counted as one byte. POSIX systems just ignore it. */
 	FILE *fp = dictopen(dict_name, "rb");
-#else
-	FILE *fp = dictopen(dict_name, "r");
-#endif
+
 	if (fp == NULL)
 		return false;
 
@@ -957,13 +773,10 @@ char *get_file_contents(const char * dict_name)
 	struct stat buf;
 	char * contents, *p;
 
-#if defined(_MSC_VER) || defined(__MINGW32__)
-	/* binary, otherwise fstat file length is confused by crlf
-	 * counted as one byte. */
+	/* On Windows, 'b' (binary mode) is mandatory, otherwise fstat file length
+	 * is confused by crlf counted as one byte. POSIX systems just ignore it. */
 	FILE *fp = dictopen(dict_name, "rb");
-#else
-	FILE *fp = dictopen(dict_name, "r");
-#endif
+
 	if (fp == NULL)
 		return NULL;
 
@@ -1002,177 +815,163 @@ char *get_file_contents(const char * dict_name)
 /* ======================================================== */
 /* Locale routines */
 
+#ifdef HAVE_LOCALE_T
+/**
+ * Create a locale object from the given locale string.
+ * @param locale Locale string, in the native OS format.
+ * @return Locale object for the given locale
+ * Note: It has to be freed by freelocale().
+ */
+locale_t newlocale_LC_CTYPE(const char *locale)
+{
+	locale_t locobj;
 #ifdef _WIN32
+		locobj = _create_locale(LC_CTYPE, locale);
+#else
+		locobj = newlocale(LC_CTYPE_MASK, locale, (locale_t)0);
+#endif /* _WIN32 */
+		return locobj;
+}
+#endif /* HAVE_LOCALE_T */
 
+/**
+ * Check that the given locale known by the system.
+ * In case we don't have locale_t, actually set the locale
+ * in order to find out if it is fine. This side effect doesn't cause
+ * harm, as the local would be set up to that value anyway shortly.
+ * @param locale Locale string
+ * @return True if known, false if unknown.
+ */
+bool try_locale(const char *locale)
+{
+#ifdef HAVE_LOCALE_T
+		locale_t ltmp = newlocale_LC_CTYPE(locale);
+		if ((locale_t)0 == ltmp) return false;
+		freelocale(ltmp);
+#else
+		lgdebug(D_USER_FILES, "Debug: Setting program's locale %s", locale);
+		if (NULL == setlocale(LC_CTYPE, locale))
+		{
+			lgdebug(D_USER_FILES, " failed!\n");
+			return false;
+		}
+		lgdebug(D_USER_FILES, ".\n");
+#endif /* HAVE_LOCALE_T */
+
+		return true;
+}
+
+/**
+ * Ensure that the program's locale has a UTF-8 codeset.
+ */
+void set_utf8_program_locale(void)
+{
+#ifndef _WIN32
+	/* The LG library doesn't use mbrtowc_l(), since it doesn't exists in
+	 * the dynamic glibc (2.22). mbsrtowcs_l() could also be used, but for
+	 * some reason it exists only in the static glibc.
+	 * In order that mbrtowc() will work for any UTF-8 character, UTF-8
+	 * codeset is ensured. */
+	const char *codeset = nl_langinfo(CODESET);
+	if (!strstr(codeset, "UTF") && !strstr(codeset, "utf"))
+	{
+		const char *locale = setlocale(LC_CTYPE, NULL);
+		/* Avoid an initial spurious message. */
+		if ((0 != strcmp(locale, "C")) && (0 != strcmp(locale, "POSIX")))
+		{
+			prt_error("Warning: Program locale %s (codeset %s) was not UTF-8; "
+						 "force-setting to en_US.UTF-8", locale, codeset);
+		}
+		locale = setlocale(LC_CTYPE, "en_US.UTF-8");
+		if (NULL == locale)
+		{
+			prt_error("Warning: Program locale en_US.UTF-8 could not be set; "
+			          "force-setting to C.UTF-8");
+			locale = setlocale(LC_CTYPE, "C.UTF-8");
+			if (NULL == locale)
+			{
+				prt_error("Warning: Could not set a UTF-8 program locale; "
+				          "program may malfunction");
+			}
+		}
+	}
+#endif /* !_WIN32 */
+}
+
+#ifdef _WIN32
 static char *
 win32_getlocale (void)
 {
-	LCID lcid;
-	LANGID langid;
-	char *ev;
-	int primary, sub;
-	char bfr[64];
-	char iso639[10];
-	char iso3166[10];
-	const char *script = NULL;
+	char lbuf[10];
+	char locale[32];
 
-	/* Let the user override the system settings through environment
-	 * variables, as on POSIX systems. Note that in GTK+ applications
-	 * since GTK+ 2.10.7 setting either LC_ALL or LANG also sets the
-	 * Win32 locale and C library locale through code in gtkmain.c.
-	 */
-	if (((ev = getenv ("LC_ALL")) != NULL && ev[0] != '\0')
-	 || ((ev = getenv ("LC_MESSAGES")) != NULL && ev[0] != '\0')
-	 || ((ev = getenv ("LANG")) != NULL && ev[0] != '\0'))
-		return safe_strdup (ev);
+	LCID lcid = GetThreadLocale();
 
-	lcid = GetThreadLocale ();
-
-	if (!GetLocaleInfo (lcid, LOCALE_SISO639LANGNAME, iso639, sizeof (iso639)) ||
-	    !GetLocaleInfo (lcid, LOCALE_SISO3166CTRYNAME, iso3166, sizeof (iso3166)))
-		return safe_strdup ("C");
-
-	/* Strip off the sorting rules, keep only the language part.	*/
-	langid = LANGIDFROMLCID (lcid);
-
-	/* Split into language and territory part.	*/
-	primary = PRIMARYLANGID (langid);
-	sub = SUBLANGID (langid);
-
-	/* Handle special cases */
-	switch (primary)
+	if (0 >= GetLocaleInfoA(lcid, LOCALE_SISO639LANGNAME, lbuf, sizeof(lbuf)))
 	{
-		case LANG_AZERI:
-			switch (sub)
-			{
-				case SUBLANG_AZERI_LATIN:
-					script = "@Latn";
-					break;
-				case SUBLANG_AZERI_CYRILLIC:
-					script = "@Cyrl";
-					break;
-			}
-			break;
-		case LANG_SERBIAN:								/* LANG_CROATIAN == LANG_SERBIAN */
-			switch (sub)
-			{
-				case SUBLANG_SERBIAN_LATIN:
-				case 0x06: /* Serbian (Latin) - Bosnia and Herzegovina */
-					script = "@Latn";
-					break;
-			}
-			break;
-		case LANG_UZBEK:
-			switch (sub)
-			{
-				case SUBLANG_UZBEK_LATIN:
-					script = "@Latn";
-					break;
-				case SUBLANG_UZBEK_CYRILLIC:
-					script = "@Cyrl";
-					break;
-			}
-			break;
+		prt_error("Error: GetLocaleInfoA LOCALE_SENGLISHLANGUAGENAME LCID=%d: "
+		          "Error %d", (int)lcid, (int)GetLastError());
+		return NULL;
 	}
+	strcpy(locale, lbuf);
+	strcat(locale, "-");
 
-	strcat (bfr, iso639);
-	strcat (bfr, "_");
-	strcat (bfr, iso3166);
+	if (0 >= GetLocaleInfoA(lcid, LOCALE_SISO3166CTRYNAME, lbuf, sizeof(lbuf)))
+	{
+		prt_error("Error: GetLocaleInfoA LOCALE_SISO3166CTRYNAME LCID=%d: "
+		          "Error %d", (int)lcid, (int)GetLastError());
+		return NULL;
+	}
+	strcat(locale, lbuf);
 
-	if (script)
-		strcat (bfr, script);
-
-	return safe_strdup (bfr);
+	return strdup(locale);
 }
-
-#endif
+#endif /* _WIN32 */
 
 char * get_default_locale(void)
 {
-	char * locale, * needle;
-
-	locale = NULL;
+	const char *lc_vars[] = {"LC_ALL", "LC_CTYPE", "LANG", NULL};
+	char *ev;
+	const char **evname;
+	char *locale = NULL;
 
+	for(evname = lc_vars; NULL != *evname; evname++)
+	{
+		ev = getenv(*evname);
+		if ((NULL != ev) && ('\0' != ev[0])) break;
+	}
+	if (NULL != *evname)
+	{
+		locale = ev;
+		lgdebug(D_USER_FILES, "Debug: Environment locale %s=%s\n", *evname, ev);
 #ifdef _WIN32
-	if(!locale)
-		locale = win32_getlocale ();
-#endif
-
-	if(!locale)
-		locale = safe_strdup (getenv ("LANG"));
-
-#if defined(HAVE_LC_MESSAGES)
-	if(!locale)
-		locale = safe_strdup (setlocale (LC_MESSAGES, NULL));
-#endif
-
-	if(!locale)
-		locale = safe_strdup (setlocale (LC_ALL, NULL));
-
-	if(!locale || strcmp(locale, "C") == 0) {
-		free(locale);
-		locale = safe_strdup("en");
+		/* If compiled with MSVC/MSYS, we still support running under Cygwin. */
+		const char *ostype = getenv("OSTYPE");
+		if ((NULL != ostype) && (0 == strcmp(ostype, "cygwin")))
+		{
+			/* Convert to Windows style locale */
+			locale = strdupa(locale);
+			locale[strcspn(locale, "_")] = '-';
+			locale[strcspn(locale, ".@")] = '\0';
+		}
+#endif /* _WIN32 */
+	}
+	else
+	{
+		lgdebug(D_USER_FILES, "Debug: Environment locale not set\n");
+#ifdef _WIN32
+		locale = win32_getlocale();
+		if (NULL == locale)
+			lgdebug(D_USER_FILES, "Debug: Cannot find user default locale\n");
+		else
+			lgdebug(D_USER_FILES, "Debug: User default locale %s\n", locale);
+		return locale; /* Already strdup'ed */
+#endif /* _WIN32 */
 	}
 
-	/* strip off "@euro" from en_GB@euro */
-	if ((needle = strchr (locale, '@')) != NULL)
-		*needle = '\0';
-
-	/* strip off ".UTF-8" from en_GB.UTF-8 */
-	if ((needle = strchr (locale, '.')) != NULL)
-		*needle = '\0';
-
-	/* strip off "_GB" from en_GB */
-	if ((needle = strchr (locale, '_')) != NULL)
-		*needle = '\0';
-
-	return locale;
+	return safe_strdup(locale);
 }
 
-/* ============================================================= */
-/* MSVC: Utilities for converting %zu to %Iu */
-
-#ifdef _MSC_VER
-int printf_compat(const char *format, ...)
-{
-	char * tmp = alloca(strlen(format)+1);
-	char * tok = tmp;
-	va_list args;
-
-	strcpy(tmp, format);
-	while ((tok = strstr(tok, "%zu"))) { tok[1] = 'I'; tok++;}
-	va_start(args, format);
-	return vprintf(tmp, args); /* fine on MSVC to return before va_end() */
-	va_end(args);
-}
-
-int fprintf_compat(FILE *file, const char *format, ...)
-{
-	char * tmp = alloca(strlen(format)+1);
-	char * tok = tmp;
-	va_list args;
-
-	strcpy(tmp, format);
-	while ((tok = strstr(tok, "%zu"))) { tok[1] = 'I'; tok++;}
-	va_start(args, format);
-	return vfprintf(file, tmp, args); /* fine on MSVC to return before va_end() */
-	va_end(args);
-}
-
-int sprintf_compat(char *str, const char *format, ...)
-{
-	char * tmp = alloca(strlen(format)+1);
-	char * tok = tmp;
-	va_list args;
-
-	strcpy(tmp, format);
-	while ((tok = strstr(tok, "%zu"))) { tok[1] = 'I'; tok++;}
-	va_start(args, format);
-	return vsprintf(str, tmp, args); /* fine on MSVC to return before va_end() */
-	va_end(args);
-}
-#endif /* _MSC_VER */
-
 /* ============================================================= */
 /* Alternatives utilities */
 
