@@ -50,7 +50,6 @@ Gword *gword_new(Sentence sent, const char *s)
 
 Gword *empty_word(void)
 {
-#ifndef MSC_VER
 	static Gword e = {
 		.subword = EMPTY_WORD_MARK,
 		.unsplit_word = &e,
@@ -58,15 +57,6 @@ Gword *empty_word(void)
 		.alternative_id = &e,
 		.status = WS_INDICT,
 	};
-#else
-	static Gword e;
-
-	e.subword = EMPTY_WORD_MARK;
-	e.unsplit_word = &e;
-	e.morpheme_type = MT_EMPTY;
-	e.alternative_id = &e;
-	e.status = WS_INDICT;
-#endif
 
 	return &e;
 }
@@ -150,7 +140,7 @@ bool wordgraph_pathpos_add(Wordgraph_pathpos **wp, Gword *p, bool used,
 	assert(NULL != p);
 
 #ifdef DEBUG
-	if (7 <= verbosity) { printf("\n"); print_hier_position(p); }
+	if (debug_level(9)) { printf("\n"); print_hier_position(p); }
 #endif
 
 	if (NULL != *wp)
@@ -761,7 +751,7 @@ static void wordgraph_show_cancel(void)
 
 #define POPEN_DOT_CMD DOT_COMMAND" "DOT_DRIVER
 #ifndef POPEN_DOT_CMD_WINDOWS
-#  if defined(_WIN32) || defined(_MSC_VER) || defined(__MINGW32__)
+#  ifdef _WIN32
 #    ifndef IMAGE_VIEWER
 #      define IMAGE_VIEWER "rundll32 PhotoViewer,ImageView_Fullscreen"
 #    endif
@@ -846,7 +836,7 @@ static void x_forkexec(const char *const argv[], pid_t *pid)
 }
 #endif /* !defined HAVE_FORK || defined POPEN_DOT */
 
-#if defined(_WIN32) || defined(_MSC_VER) || defined(__MINGW32__)
+#ifdef _WIN32
 #define TMPDIR (getenv("TEMP") ? getenv("TEMP") : ".")
 #else
 #define TMPDIR (getenv("TMPDIR") ? getenv("TMPDIR") : "/tmp")
